/**
 *    @file  clustercon.c
 *   @brief
 *
 *  <+DETAILED+>
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  25/10/2024
 *      Revision:  none
 * Last modified:  2024-11-13 21:40
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "clustercon.h"
#include "base64.h"                             /* Base64 encoder and decoder */

#include <unistd.h>
#include <string.h>
#ifdef _WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#else
#include <arpa/inet.h>
#endif
#include <openssl/err.h>

typedef struct rsclustercon_s {
    char*              host;
    char*              user;
    char*              pass;
    unsigned short int insecure;
    char*              cacert;
    int                sock;
    SSL_CTX*           ctx;
    SSL*               ssl;
} rsclustercon_t;

rsclustercon_t*    cluster_new(const cluster_t* cluster) {
    rsclustercon_t* rsclustercon = NULL;

    if (NULL==(rsclustercon=malloc(sizeof(struct rsclustercon_s)))) {
        perror("cluster_new");
        return NULL;
    }
    if (NULL==(rsclustercon->host=strdup(cluster->host))) {
        perror("cluster_new host");
        free(rsclustercon);
        return NULL;
    }
    if (NULL==(rsclustercon->user=strdup(cluster->user))) {
        perror("cluster_new user");
        free(rsclustercon->host);
        free(rsclustercon);
        return NULL;
    }
    if (NULL==(rsclustercon->pass=strdup(cluster->pass))) {
        perror("cluster_new pass");
        free(rsclustercon->user);
        free(rsclustercon->host);
        free(rsclustercon);
        return NULL;
    }
    if (NULL==(rsclustercon->cacert=strdup(cluster->cacert))) {
        perror("cluster_new cacert");
        free(rsclustercon->pass);
        free(rsclustercon->user);
        free(rsclustercon->host);
        free(rsclustercon);
        return NULL;
    }
    rsclustercon->insecure = strcmp(cluster->insecure,"false");
    return rsclustercon;
}

int   cluster_open(rsclustercon_t* rsclustercon) {
    struct sockaddr_in server_addr;

    /* Socket creation */
    rsclustercon->sock = socket(AF_INET, SOCK_STREAM, 0);
    if (rsclustercon->sock < 0) {
        perror("Socket creation error");
        return 1;
    }

    server_addr.sin_family = AF_INET;
    server_addr.sin_port = htons(9443);
    if (inet_pton(AF_INET, rsclustercon->host, &server_addr.sin_addr) <= 0) {
        perror("Invalid address");
        return 2;
    }

    /* Server connection */
    if (connect(rsclustercon->sock, (struct sockaddr *)&server_addr, sizeof(server_addr)) != 0) {
        perror("Connection failed");
        return 3;
    }

    /* OpenSSL initialization */
    SSL_library_init();
    /* OpenSSL_add_all_algorithms(); */
    OpenSSL_add_ssl_algorithms();
    SSL_load_error_strings();

    /* Initialize OpenSSL context */
    rsclustercon->ctx = SSL_CTX_new(SSLv23_client_method());
    if (!rsclustercon->ctx) {
        ERR_print_errors_fp(stderr);
        return 4;
    }

    /* Load server certificate or certificate from a certification authority (CA) */
    if (strcmp("",rsclustercon->cacert)&&rsclustercon->insecure==0)
        if (SSL_CTX_load_verify_locations(rsclustercon->ctx, rsclustercon->cacert, NULL) != 1) {
            ERR_print_errors_fp(stderr);
            return 5;
        }

    if (rsclustercon->insecure)
        /* Disable certificat check */
        SSL_CTX_set_verify(rsclustercon->ctx, SSL_VERIFY_NONE, NULL);
    else
        /* Enable certificat check */
        SSL_CTX_set_verify(rsclustercon->ctx, SSL_VERIFY_PEER, NULL);


    /* Link SSL configuration to the socket */
    rsclustercon->ssl = SSL_new(rsclustercon->ctx);
    SSL_set_fd(rsclustercon->ssl, rsclustercon->sock);

    /* Initiate SSL connection */
    if (SSL_connect(rsclustercon->ssl) <= 0) {
        ERR_print_errors_fp(stderr);
        return 6;
    }

    return 0;
}

cJSON* cluster_queryget(const rsclustercon_t* rsclustercon, const char* endpoint) {
    char buf[1024];

    /* Prepare basic authentication */
    char* auth_encoded;
    {
        char* auth_clear;
        if (NULL==(auth_clear = (char*)malloc(strlen(rsclustercon->user)+1+strlen(rsclustercon->pass)+1))) {
            perror("cluster_queryget malloc(auth)");
            return NULL;
        };
        strcpy(auth_clear,rsclustercon->user);
        strcat(auth_clear,":");
        strcat(auth_clear,rsclustercon->pass);
        auth_encoded=base64_encode(auth_clear);
        free(auth_clear);
    }

    /* Prepare query */
    char http_request[512];
    snprintf(http_request, sizeof(http_request),
             "GET %s HTTP/1.1\r\n"
             "Host: %s\r\n"
             "Content-Type: application/json\r\n"
             "Authorization: Basic %s\r\n"
             "Connection: close\r\n\r\n",
             endpoint,rsclustercon->host,auth_encoded
            );
    free(auth_encoded);

    /* Send query */
    SSL_write(rsclustercon->ssl, http_request, strlen(http_request));

    /* Read reply */
    int bytes=0;
    char* reply=NULL;
    if (NULL==(reply=strdup(""))) {
        perror("cluster_queryget strdup");
        return NULL;
    };
    while ((bytes=SSL_read(rsclustercon->ssl, buf, 1024 - 1)) > 0) {
        buf[bytes] = 0;
        char* newreply;
        if ((newreply=(char*)realloc(reply,strlen(reply)+bytes+1))==NULL) {
            perror("Unable to allocate reply buffer");
            free(reply);
            return NULL;
        } else
            reply = newreply;
        strcat(reply, buf);
    }

    /* Remove HTTP headers */
    char* http_body;
    char* retval_txt;
    if (NULL==(http_body = strstr(reply,"\r\n\r\n")))
        retval_txt= NULL;
    else
        retval_txt=strdup(http_body);
    free(reply);

    cJSON* retval_json = cJSON_Parse(retval_txt);
    free(retval_txt);

    return retval_json;
}

void  cluster_close(rsclustercon_t* rsclustercon) {
    SSL_shutdown(rsclustercon->ssl);
    SSL_free(rsclustercon->ssl);
    close(rsclustercon->sock);
    SSL_CTX_free(rsclustercon->ctx);
    EVP_cleanup();
}

void  cluster_del(rsclustercon_t* rsclustercon) {
    free(rsclustercon->cacert);
    free(rsclustercon->pass);
    free(rsclustercon->user);
    free(rsclustercon->host);
    free(rsclustercon);
}

/* vim: set tw=80: */
