/**
 *    @file  libhttp.c
 *   @brief  HTTP parsing and building library
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  15/11/2024
 *      Revision:  none
 * Last modified:  2024-11-30 22:15
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "libhttp.h"
#include "sclist.h"
#include "base64.h"                             /* Base64 encoder and decoder */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

typedef struct HTTP_s {
    sclist_t* headers;
    char* body;
} HTTP_t;

typedef struct HTTPHeader_s {
    sclistrecord_t* self;
    char* name;
    char* value;
} HTTPHeader_t;

HTTPHeader_t* HTTPHeader_setname(HTTPHeader_t* header, const char* name) {
    char* newname;

    assert(header);
    assert(header->name);
    assert(header->value);
    assert(name);

    if (name[0]==0) {
        fprintf(stderr,"ERROR: HTTP_setname empty name\n");
        return NULL;
    }

    if (NULL==(newname=realloc(header->name,strlen(name)+1))) {
        perror("ERROR: HTTPHeader_setname realloc");
        return NULL;
    }
    header->name = newname;
    strcpy(header->name,name);
    return header;
}

char* HTTPHeader_getname(HTTPHeader_t* header) {
    assert(header);
    assert(header->name);
    assert(header->value);

    return header->name;
}

HTTPHeader_t* HTTPHeader_setvalue(HTTPHeader_t* header, const char* value) {
    char* newvalue;

    assert(header);
    assert(header->name);
    assert(header->value);
    assert(value);

    if (NULL==(newvalue=realloc(header->value,strlen(value)+1))) {
        perror("ERROR: HTTPHeader_setvalue realloc");
        return NULL;
    }
    header->value = newvalue;
    strcpy(header->value,value);
    return header;
}

char* HTTPHeader_getvalue(HTTPHeader_t* header) {
    assert(header);
    assert(header->name);
    assert(header->value);

    return header->value;
}

static HTTPHeader_t* HTTPHeader_new(const char* name, const char* value) {
    HTTPHeader_t* header;

    assert(name);
    assert(value);

    if (NULL==(header=malloc(sizeof(struct HTTPHeader_s)))) {
        perror("ERROR: HTTPHeader_new");
        return NULL;
    }
    if (NULL==(header->name=strdup(name))) {
        perror("ERROR: HTTPHeader_new name");
        free(header);
        return NULL;
    }
    if (NULL==(header->value=strdup(value))) {
        perror("ERROR: HTTPHeader_new value");
        free(header->name);
        free(header);
        return NULL;
    }
    header->self = NULL;
    return header;
}

static void HTTPHeader_del(HTTPHeader_t* header) {
    assert(header);
    assert(header->name);
    assert(header->value);

    free(header->name);
    free(header->value);
    free(header);
}

HTTP_t* HTTP_new() {
    HTTP_t* http;

    if (NULL==(http=malloc(sizeof(struct HTTP_s)))) {
        perror("ERROR: HTTP_new HTTP_s");
        return NULL;
    }

    if (NULL==(http->body=strdup(""))) {
        perror("ERROR: HTTP_new body");
        free(http);
        return NULL;
    };

    if (NULL==(http->headers = sclist_new())) {
        fprintf(stderr,"ERROR: HTTP_new headers\n");
        free(http->body);
        free(http);
        return NULL;
    }

    return http;
}

void HTTP_del(HTTP_t* http) {
    sclistrecord_t* headerlst_entry;

    assert(http);
    assert(http->headers);
    assert(http->body);

    headerlst_entry = sclist_firstrecord(http->headers);
    while (headerlst_entry) {
        HTTPHeader_t* headerlst_value;

        /* Free payload */
        headerlst_value=sclist_getvalue(headerlst_entry);
        HTTPHeader_del(headerlst_value);

        /* Remember the entry to delete before moving to the next one */
        sclistrecord_t* tmp = headerlst_entry;
        headerlst_entry = sclist_nextrecord(headerlst_entry);
        sclist_remrecord(http->headers,tmp);
    }
    sclist_del(http->headers);

    free(http->body);
    free(http);
}

HTTP_t* HTTP_setbody(HTTP_t* http, const char* body) {
    char* newbody;

    assert(http);
    assert(http->headers);
    assert(http->body);
    assert(body);

    if (NULL==(newbody=realloc(http->body,strlen(body)+1))) {
        perror("ERROR: HTTP_setbody realloc");
        return NULL;
    }

    http->body = newbody;
    strcpy(http->body, body);
    return http;
}

char* HTTP_getbody(HTTP_t* http) {
    assert(http);
    assert(http->headers);
    assert(http->body);

    return http->body;
}

HTTPHeader_t* HTTP_addheader(HTTP_t* http, const char* name, const char* value) {
    HTTPHeader_t* header;

    assert(http);
    assert(http->headers);
    assert(http->body);
    assert(name);
    assert(value);

    if (name[0]==0) {
        fprintf(stderr,"ERROR: HTTP_addheader empty name\n");
        return NULL;
    }

    if (NULL==(header = HTTPHeader_new(name,value))) {
        fprintf(stderr,"ERROR: HTTP_addheader HTTPHeader_new\n");
        return NULL;
    }

    if (NULL==(header->self=sclist_addrecord(http->headers,header))) {
        fprintf(stderr,"ERROR: HTTP_addheader\n");
        HTTPHeader_del(header);
        return NULL;
    }
    return header;
}

HTTPHeader_t* HTTP_addbasicauth(HTTP_t* http, const char* login, const char* pass) {
    char* auth_encoded;
    char* auth;
    HTTPHeader_t* header;

    assert(http);
    assert(http->headers);
    assert(http->body);
    assert(login);
    assert(pass);

    if (NULL==(auth=malloc(strlen(login)+1+strlen(pass)+1))) {
        perror("ERROR: HTTP_addbasicauth credentials");
        return NULL;
    }
    /* strcpy/strcat expected to be faster than sprintf */
    strcpy(auth,login);
    strcat(auth,":");
    strcat(auth,pass);
    auth_encoded=base64_encode(auth);
    free(auth);

    if (NULL==(auth=malloc(strlen("Basic ")+strlen(auth_encoded)+1))) {
        perror("ERROR: HTTP_addbasicauth value");
        free(auth_encoded);
        return NULL;
    }
    strcpy(auth,"Basic ");
    strcat(auth, auth_encoded);
    free(auth_encoded);

    if (NULL==(header=HTTP_addheader(http,"Authorization",auth))) {
        fprintf(stderr,"ERROR: HTTP_addbasicauth addheader\n");
        free(auth);
        return NULL;
    }
    free(auth);
    return header;
}

HTTPHeader_t* HTTP_firstheader(const HTTP_t* http) {
    sclistrecord_t* headerlst_entry;

    assert(http);
    assert(http->headers);
    assert(http->body);

    if (NULL==(headerlst_entry=sclist_firstrecord(http->headers)))
        return NULL;
    return sclist_getvalue(headerlst_entry);
}

HTTPHeader_t* HTTP_nextheader(const HTTPHeader_t* header) {
    sclistrecord_t* headerlst_entry;

    assert(header);
    assert(header->self);
    assert(header->name);
    assert(header->value);

    headerlst_entry = header->self;

    /* EOL reached */
    if (NULL==(headerlst_entry = sclist_nextrecord(headerlst_entry)))
        return NULL;
    else
        return sclist_getvalue(headerlst_entry);
}

HTTPHeader_t* HTTP_findheader(const HTTPHeader_t* start, const char* name) {
    sclistrecord_t* headerlst_entry;

    assert(start);
    assert(start->self);
    assert(start->name);
    assert(start->value);
    assert(name);

    headerlst_entry = start->self;

    while (headerlst_entry) {
        HTTPHeader_t* headerlst_value;
        char* header_name;

        headerlst_value = sclist_getvalue(headerlst_entry);
        assert(headerlst_value);
        header_name = HTTPHeader_getname(headerlst_value);
        if (0==strcmp(header_name,name))
            break;
        headerlst_entry=sclist_nextrecord(headerlst_entry);
    }
    if (!headerlst_entry)
        /* Not found */
        return NULL;
    else
        /* Found */
        return sclist_getvalue(headerlst_entry);
}


HTTP_t* HTTP_remheader(HTTP_t* http, HTTPHeader_t* header) {
    assert(http);
    assert(http->headers);
    assert(http->body);
    assert(header);
    assert(header->self);
    assert(header->name);
    assert(header->value);

    /* Remove from the header list */
    if (NULL==sclist_remrecord(http->headers, header->self)) {
        fprintf(stderr,"ERROR: HTTP_remheader not found\n");
        return NULL;
    }
    header->self = NULL;
    HTTPHeader_del(header);

    return http;
}

static char* strconcat(char* dst, const char* src) {
    char* newdst;

    assert(dst);
    assert(src);

    if (NULL==(newdst=realloc(dst,strlen(dst)+strlen(src)+1))) {
        perror("ERROR: strconcat");
        return NULL;
    }
    dst = newdst;
    strcat(dst,src);
    return dst;
}

char* HTTP_buildheaders(const HTTP_t* http) {
    char* headers_str=NULL;
    sclistrecord_t* headerlst_entry;

    assert(http);
    assert(http->headers);
    assert(http->body);

    if (NULL==(headers_str=strdup(""))) {
        perror("ERROR: HTTP_buildheaders headers_str");
        return NULL;
    }
    for (
        headerlst_entry=sclist_firstrecord(http->headers);
        headerlst_entry;
        headerlst_entry = sclist_nextrecord(headerlst_entry)
    ) {
        HTTPHeader_t* headerlst_value;

        headerlst_value = sclist_getvalue(headerlst_entry);
        assert(headerlst_value);
        /** @bug possible memory leak, check strconcat return values */
        headers_str=strconcat(headers_str,HTTPHeader_getname(headerlst_value));
        headers_str=strconcat(headers_str,": ");
        headers_str=strconcat(headers_str,HTTPHeader_getvalue(headerlst_value));
        headers_str=strconcat(headers_str,"\r\n");
    }

    return headers_str;
}

/* @todo redesign headers and implementation to identify different errors */
char* HTTP_buildrequest(const HTTP_t* http, const HTTPMethod_t method, const char* uri, const HTTPVersion_t version) {
    char* request;
    char* newrequest;
    char* headers;

    assert(uri);

    if (NULL==(request=strdup(""))) {
        perror("ERROR: HTTP_getrequest strdup");
        return NULL;
    }

    switch(method) {
    case HTTPMETHOD_GET:
        if (NULL==(newrequest=strconcat(request, "GET"))) {
            fprintf(stderr,"ERROR: HTTP_buildrequest method\n");
            free(request);
            return NULL;
        }
        request = newrequest;
        break;;
    case HTTPMETHOD_HEAD:
    case HTTPMETHOD_POST:
    case HTTPMETHOD_PUT:
    case HTTPMETHOD_DELETE:
    case HTTPMETHOD_CONNECT:
    case HTTPMETHOD_OPTIONS:
    case HTTPMETHOD_TRACE:
    case HTTPMETHOD_PATCH:
    case HTTPMETHOD_INVALID:
    default:
        fprintf(stderr,"ERROR: HTTP method not supported\n");
        free(request);
        return NULL;
        break;;
    }

    if (uri[0]) {
        if (NULL==(newrequest=strconcat(request, " "))) {
            fprintf(stderr,"ERROR: HTTP_buildrequest uri separator\n");
            free(request);
            return NULL;
        }
        request = newrequest;
        if (NULL==(newrequest=strconcat(request, uri))) {
            fprintf(stderr,"ERROR: HTTP_buildrequest uri\n");
            free(request);
            return NULL;
        }
        request = newrequest;
    }

    if (NULL==(newrequest=strconcat(request, " HTTP/"))) {
        fprintf(stderr,"ERROR: HTTP_buildrequest HTTP\n");
        free(request);
        return NULL;
    }
    request = newrequest;
    switch(version) {
    case HTTPVERSION_HTTP11:
    case HTTPVERSION_HTTP11b:
        if (NULL==(newrequest=strconcat(request, "1.1"))) {
            fprintf(stderr,"ERROR: HTTP_buildrequest Version\n");
            free(request);
            return NULL;
        }
        request = newrequest;
        break;;
    case HTTPVERSION_HTTP09:
    case HTTPVERSION_HTTP10:
    case HTTPVERSION_HTTP2:
    case HTTPVERSION_HTTP3:
    case HTTPVERSION_INVALID:
    default:
        fprintf(stderr,"ERROR: HTTP version not supported\n");
        free(request);
        return NULL;
        break;;
    }
    if (NULL==(newrequest=strconcat(request, "\r\n"))) {
        fprintf(stderr,"ERROR: HTTP_buildrequest version separator\n");
        free(request);
        return NULL;
    }
    request = newrequest;

    if (NULL==(headers=HTTP_buildheaders(http))) {
        fprintf(stderr,"ERROR: HTTP_buildrequest buildheaders\n");
        free(request);
        return NULL;
    }

    if (NULL==(newrequest=strconcat(request, headers))) {
        fprintf(stderr,"ERROR: HTTP_buildrequest headers\n");
        free(request);
        return NULL;
    }
    request = newrequest;
    free(headers);

    if (NULL==(newrequest=strconcat(request, "\r\n"))) {
        fprintf(stderr,"ERROR: HTTP_buildrequest header separator\n");
        free(request);
        return NULL;
    }
    request = newrequest;

    if (NULL==(newrequest=strconcat(request, http->body))) {
        fprintf(stderr,"ERROR: HTTP_buildrequest body\n");
        free(request);
        return NULL;
    }
    request = newrequest;

    return request;
}

char*         HTTP_buildreply(const HTTP_t* http, const HTTPVersion_t version, const HTTPStatus_t status) {
    char* reply;
    char* newreply;
    char* headers;
    char status_str[6];

    if (NULL==(reply=strdup(""))) {
        fprintf(stderr,"ERROR: HTTP_buildreply strdup\n");
        return NULL;
    }

    if (NULL==(newreply=strconcat(reply, "HTTP/"))) {
        fprintf(stderr,"ERROR: HTTP_buildreply HTTP\n");
        free(reply);
        return NULL;
    }
    reply = newreply;
    switch(version) {
    case HTTPVERSION_HTTP11:
    case HTTPVERSION_HTTP11b:
        if (NULL==(newreply=strconcat(reply, "1.1"))) {
            fprintf(stderr,"ERROR: HTTP_buildreply Version\n");
            free(reply);
            return NULL;
        }
        reply = newreply;
        break;;
    case HTTPVERSION_HTTP09:
    case HTTPVERSION_HTTP10:
    case HTTPVERSION_HTTP2:
    case HTTPVERSION_HTTP3:
    case HTTPVERSION_INVALID:
    default:
        fprintf(stderr,"ERROR: HTTP version not supported\n");
        free(reply);
        return NULL;
        break;;
    }

    snprintf(status_str,6," %d ",status);
    if (NULL==(newreply=strconcat(reply, status_str))) {
        fprintf(stderr,"ERROR: HTTP_buildreply status code\n");
        free(reply);
        return NULL;
    }
    reply = newreply;

    if (NULL==(newreply=strconcat(reply, "Dummy Reason"))) {
        fprintf(stderr,"ERROR: HTTP_buildreply reason phrase\n");
        free(reply);
        return NULL;
    }
    reply = newreply;

    if (NULL==(newreply=strconcat(reply, "\r\n"))) {
        fprintf(stderr,"ERROR: HTTP_buildreply reason separator\n");
        free(reply);
        return NULL;
    }
    reply = newreply;

    if (NULL==(headers=HTTP_buildheaders(http))) {
        fprintf(stderr,"ERROR: HTTP_buildreply buildheaders\n");
        free(reply);
        return NULL;
    }

    if (NULL==(newreply=strconcat(reply, headers))) {
        fprintf(stderr,"ERROR: HTTP_buildreply headers\n");
        free(reply);
        return NULL;
    }
    reply = newreply;
    free(headers);

    if (NULL==(newreply=strconcat(reply, "\r\n"))) {
        fprintf(stderr,"ERROR: HTTP_buildreply header separator\n");
        free(reply);
        return NULL;
    }
    reply = newreply;

    if (NULL==(newreply=strconcat(reply, http->body))) {
        fprintf(stderr,"ERROR: HTTP_buildreply body\n");
        free(reply);
        return NULL;
    }
    reply = newreply;

    return reply;
}

HTTP_t*       HTTP_parseheaders(const char* text) {
    (void) text;
    fprintf(stderr,"Not yet implemented\n");
    abort();
    return NULL;
}

void          HTTP_parserequest(const char* request, HTTP_t** http, HTTPMethod_t method, char* uri, HTTPVersion_t version) {
    (void) request;
    (void) http;
    (void) method;
    (void) uri;
    (void) version;
    fprintf(stderr,"Not yet implemented\n");
    abort();
}

void          HTTP_parsereply(const char* reply, HTTP_t** http, HTTPVersion_t* version, HTTPStatus_t* status) {
    (void) reply;
    (void) http;
    (void) version;
    (void) status;
    fprintf(stderr,"Not yet implemented\n");
    abort();
}
