/**
 *    @file  libhttp.h
 *   @brief  HTTP parsing and building library
 *
 * HTTP Status code reference : https://en.wikipedia.org/wiki/List_of_HTTP_status_codes
 * HTTP Header fields reference : https://en.wikipedia.org/wiki/List_of_HTTP_header_fields
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  19/11/2024
 *      Revision:  none
 * Last modified:  2024-11-29 21:52
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __LIBHTTP_H__
#define __LIBHTTP_H__

typedef enum HTTPVersion_e {
    HTTPVERSION_HTTP09,
    HTTPVERSION_HTTP10,
    HTTPVERSION_HTTP11,
    HTTPVERSION_HTTP11b,
    HTTPVERSION_HTTP2,
    HTTPVERSION_HTTP3,
    HTTPVERSION_INVALID
} HTTPVersion_t;

typedef enum HTTPMethod_e {
    HTTPMETHOD_GET,
    HTTPMETHOD_HEAD,
    HTTPMETHOD_POST,
    HTTPMETHOD_PUT,
    HTTPMETHOD_DELETE,
    HTTPMETHOD_CONNECT,
    HTTPMETHOD_OPTIONS,
    HTTPMETHOD_TRACE,
    HTTPMETHOD_PATCH,
    HTTPMETHOD_INVALID
} HTTPMethod_t;

/* https://en.wikipedia.org/wiki/List_of_HTTP_status_codes */
typedef enum   HTTPStatus_e {
    INFO_CONTINUE = 100,
    INFO_SWITCHINGPROTOCOLS = 101,
    INFO_PROCESSING = 102,
    INFO_EARLYHINTS = 103,
    SUCCESS_OK = 200,
    SUCCESS_CREATED = 201,
    SUCCESS_ACCEPTED = 202,
    SUCCESS_NONAUTHORITATIVEINFORMATION = 203,
    SUCCESS_NOCONTENT = 204,
    SUCCESS_RESETCONTENT = 205,
    SUCCESS_PARTIALCONTENT = 206,
    SUCCESS_MULTISTATUS = 207,
    SUCCESS_ALREADYREPORTED = 208,
    SUCCESS_IMUSED = 226,
    REDIRECTION_MULTIPLECHOICES = 300,
    REDIRECTION_MOVEDPERMANENTLY = 301,
    REDIRECTION_FOUND = 302, /* Moved temporarily */
    REDIRECTION_SEEOTHER = 303,
    REDIRECTION_NOTMODIFIED = 304,
    REDIRECTION_USEPROXY = 305,
    REDIRECTION_SWITCHPROXY = 306,
    REDIRECTION_TEMPORARYREDIRECT = 307,
    REDIRECTION_PERMANENTREDIRECT = 308,
    CLIENTERROR_BADREQUEST = 400,
    CLIENTERROR_UNAUTHORIZED = 401,
    CLIENTERROR_PAYMENTREQUIRED = 402,
    CLIENTERROR_FORBIDDEN = 403,
    CLIENTERROR_NOTFOUND = 404,
    CLIENTERROR_METHODNOTALLOWED = 405,
    CLIENTERROR_NOTACCEPTABLE = 406,
    CLIENTERROR_PROXYAUTHENTICATIONREQUIRED = 407,
    CLIENTERROR_REQUESTTIMEOUT = 408,
    CLIENTERROR_CONFLICT = 409,
    CLIENTERROR_GONE = 410,
    CLIENTERROR_LENGTHREQUIRED = 411,
    CLIENTERRORPRECONDITIONFAILED = 412,
    CLIENTERROR_PAYLOADTOOLARGE = 413,
    CLIENTERROR_URITOOLONG = 414,
    CLIENTERROR_UNSUPPORTEDMEDIATYPE = 415,
    CLIENTERROR_RANGENOTSATISFIABLE = 416,
    CLIENTERROR_EXPECTATIONFAILED = 417,
    CLIENTERROR_IMATEAPOT = 418,
    CLIENTERROR_MISDIRECTEDREQUEST = 421,
    CLIENTERROR_UNPROCESSABLECONTENT = 422,
    CLIENTERROR_LOCKED = 423,
    CLIENTERROR_FAILEDDEPENDENCY = 424,
    CLIENTERROR_TOOEARLY = 425,
    CLIENTERROR_UPGRADEREQUIRED = 426,
    CLIENTERROR_PRECONDITIONREQUIRED = 428,
    CLIENTERROR_TOOMANYREQUESTS = 429,
    CLIENTERROR_REQUESTHEADERFIELDSTOOLARGE = 431,
    CLIENTERROR_UNAVAILABLEFORLEGALREASONS = 451,
    SERVERERROR_INTERNALSERVERERROR = 500,
    SERVERERROR_NOTIMPLEMENTED = 501,
    SERVERERROR_BADGATEWAY = 502,
    SERVERERROR_SERVICEUNAVAILABLE = 503,
    SERVERERROR_GATEWAYTIMEOUT = 504,
    SERVERERROR_HTTPVERSIONNOTSUPPORTED = 505,
    SERVERERROR_VARIANTALSONEGOTIATES = 506,
    SERVERERROR_INSUFFICIENTSTORAGE = 507,
    SERVERERROR_LOOPDETECTED = 508,
    SERVERERROR_NOTEXTENDED = 510,
    SERVERERROR_NETWORKAUTHENTICATIONREQUIRED = 511
} HTTPStatus_t;

typedef struct HTTPHeader_s  HTTPHeader_t;

typedef struct HTTP_s HTTP_t;

HTTP_t*       HTTP_new();
void          HTTP_del(HTTP_t* http);
HTTP_t*       HTTP_setbody(HTTP_t* http, const char* body);
char*         HTTP_getbody(HTTP_t* http);
HTTPHeader_t* HTTP_addheader(HTTP_t* http, const char* name, const char* value);
HTTPHeader_t* HTTP_addbasicauth(HTTP_t* http, const char* login, const char* pass);
HTTPHeader_t* HTTPHeader_setname(HTTPHeader_t* header, const char* name);
char*         HTTPHeader_getname(HTTPHeader_t* header);
HTTPHeader_t* HTTPHeader_setvalue(HTTPHeader_t* header, const char* value);
char*         HTTPHeader_getvalue(HTTPHeader_t* header);
HTTPHeader_t* HTTP_firstheader(const HTTP_t* http);
HTTPHeader_t* HTTP_nextheader(const HTTPHeader_t* header);
HTTPHeader_t* HTTP_findheader(const HTTPHeader_t* start, const char* name);
HTTP_t*       HTTP_remheader(HTTP_t* http, HTTPHeader_t* header);

char*         HTTP_buildheaders(const HTTP_t* http);
HTTP_t*       HTTP_parseheaders(const char* text);

char*         HTTP_buildrequest(const HTTP_t* http, const HTTPMethod_t method, const char* uri, const HTTPVersion_t version);
void          HTTP_parserequest(const char* request, HTTP_t** http, HTTPMethod_t method, char* uri, HTTPVersion_t version);

char*         HTTP_buildreply(const HTTP_t* http, const HTTPVersion_t version, const HTTPStatus_t status);
void          HTTP_parsereply(const char* reply, HTTP_t** http, HTTPVersion_t* version, HTTPStatus_t* status);


#endif /* __LIBHTTP_H__ */
