/**
 *    @file  rptbdbs.c
 *   @brief
 *
 *  <+DETAILED+>
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  28/10/2024
 *      Revision:  none
 * Last modified:  2024-11-06 19:21
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "rptbdbs.h"
#include "clustercon.h"
#include "json.h"
#include "csv.h"
#include <string.h>

/* To refactor in a report common file */
static void csv_addjsonfield(FILE* reportfile, const cJSON* json, char* fieldname) {
    char* text = json2text(cJSON_GetObjectItemCaseSensitive(json, fieldname));
    csv_addfield(reportfile,text);
    free(text);
}

void report_bdbs_header(FILE* reportfile) {
    fprintf(reportfile,
            "cluster_host,uid,name,shards_count,replication,data_persistence,memory_size,used_memory,module_list\r\n"
           );
}

static cJSON* report_querygetjson(const cluster_t* cluster, const char* endpoint) {
    rsclustercon_t* rsclustercon;

    if (NULL==(rsclustercon = cluster_new(cluster))) {
        fprintf(stderr,"report_querygetjson cluster_new failed\n");
        return NULL;
    }
    if (0!=cluster_open(rsclustercon)) {
        fprintf(stderr,"report_querygetjson cluster_open failed\n");
        cluster_del(rsclustercon);
        return NULL;
    }

    cJSON* retval;
    if (NULL==(retval = cluster_queryget(rsclustercon, endpoint))) {
        const char *error_ptr = cJSON_GetErrorPtr();
        if (error_ptr != NULL)
            fprintf(stderr, "Error before: %s\n", error_ptr);
        retval = cJSON_CreateObject();
    }

    cluster_close(rsclustercon);
    cluster_del(rsclustercon);

    return retval;
}

void report_bdbs(FILE* reportfile, const cluster_t* cluster) {
    cJSON* bdbs_json;
    cJSON* bdbsstats_json;

    bdbs_json = report_querygetjson(cluster, "/v1/bdbs");
    bdbsstats_json = report_querygetjson(cluster, "/v1/bdbs/stats/last");

    const cJSON* bdb_json;
    cJSON_ArrayForEach(bdb_json, bdbs_json) {
        char* uid=json2text(cJSON_GetObjectItemCaseSensitive(bdb_json, "uid"));
        cJSON* stats_json = cJSON_GetObjectItemCaseSensitive(bdbsstats_json, uid);
        free(uid);
        csv_addfield(reportfile,cluster->host);
        csv_addjsonfield(reportfile, bdb_json, "uid");
        csv_addjsonfield(reportfile, bdb_json, "name");
        csv_addjsonfield(reportfile, bdb_json, "shards_count");
        csv_addjsonfield(reportfile, bdb_json, "replication");
        csv_addjsonfield(reportfile, bdb_json, "data_persistence");
        csv_addjsonfield(reportfile, bdb_json, "memory_size");
        csv_addjsonfield(reportfile, stats_json, "used_memory");

        /* Iterate the module list and build the text list */
        char* modlst;
        if (NULL==(modlst=strdup(""))) {
            perror("rptbdbs repport_bdbs module_list");
            exit(EXIT_FAILURE);
        }

        const cJSON* module_json;
        cJSON_ArrayForEach(module_json, cJSON_GetObjectItemCaseSensitive(bdb_json, "module_list")) {
            char* modulename = cJSON_GetObjectItemCaseSensitive(module_json, "module_name")->valuestring;
            char* newmodlst;
            if (NULL==(newmodlst = (char*)realloc(modlst,strlen(modlst)+strlen(modulename?modulename:"")+2+1))) {
                perror("rptbdbs repport_bdbs module_list");
                free(modlst);
                return;
            } else
                modlst = newmodlst;
            strcat(modlst,(modulename?modulename:""));
            strcat(modlst,", ");
        }

        /* Remove last comma if applicable */
        if (strlen(modlst)>2)
            modlst[strlen(modlst)-2]=0;
        csv_addfield(reportfile,modlst);
        free(modlst);

        csv_addline(reportfile);
    }
    cJSON_Delete(bdbs_json);
    cJSON_Delete(bdbsstats_json);
}
/* vim: set tw=80: */
