/**   @file  suite_base64.c
 *   @brief  Check testsuite for base64 encoding and decoding functions
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-11-11 21:33
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <check.h>
#include <stdlib.h>
#include "checktools.inc"

#include "base64.h"

/** Dummy test to trigger fixtures
 *
 * This test is defined and added to all testcases in order to execute at
 * least one test test in each testcase of the testsuite, trigger the
 * fixtures and have a decent coverage report.
 */
START_TEST(base64_test) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    abort();
}
END_TEST

START_TEST(base64_encode_null) {
    /* Should trigger SIG11  */
    forktest_only;
    base64_encode(NULL);
}
END_TEST

START_TEST(base64_encode_empty) {
    char* enc = NULL;
    enc = base64_encode("");
    ck_assert(NULL!=enc);
    ck_assert(0==strcmp(enc,""));
    free(enc);
}
END_TEST

START_TEST(base64_encode_abc) {
    char* enc = NULL;
    enc = base64_encode("abc");
    ck_assert(NULL!=enc);
    ck_assert(0==strcmp(enc,"YWJj"));
    free(enc);
}
END_TEST

START_TEST(base64_decode_null) {
    /* Should trigger SIG11  */
    forktest_only;
    base64_decode(NULL);
}
END_TEST

START_TEST(base64_decode_empty) {
    char* dec = NULL;
    dec = base64_decode("");
    ck_assert(NULL!=dec);
    ck_assert(0==strcmp(dec,""));
    free(dec);
}
END_TEST

START_TEST(base64_decode_abc) {
    char* dec = NULL;
    dec = base64_decode("YWJj");
    ck_assert(NULL!=dec);
    ck_assert(0==strcmp(dec,"abc"));
    free(dec);
}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void base64_checked_uninitialized_setup() {
    signals_catch();
    forktest_gprofdir();
}

/* End of test (potentially in child) */
void base64_checked_uninitialized_teardown() {
    signals_release();
}
/************************************************************************/
void base64_unchecked_common_setup() {
    forktest_init();
}
void base64_unchecked_common_teardown() {
}
/************************************************************************/
Suite* base64_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("Base64");

    tc = tcase_create("Base64Encode");
    tcase_set_tags(tc,"Base64Tag Base64Tag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, base64_checked_uninitialized_setup, base64_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, base64_unchecked_common_setup, base64_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, base64_test,6);
    tcase_add_test_raise_signal(tc, base64_encode_null,11);
    tcase_add_test(tc, base64_encode_empty);
    tcase_add_test(tc, base64_encode_abc);

    tc = tcase_create("Base64Decode");
    tcase_set_tags(tc,"Base64Tag Base64Tag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, base64_checked_uninitialized_setup, base64_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, base64_unchecked_common_setup, base64_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, base64_test,6);
    tcase_add_test_raise_signal(tc, base64_decode_null,11);
    tcase_add_test(tc, base64_decode_empty);
    tcase_add_test(tc, base64_decode_abc);

    return s;
}
/* vim: set tw=80: */
