/**   @file  suite_clusterlst.c
 *   @brief  Check testsuite for clusterlst functions
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-11-19 18:52
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <check.h>
#include <stdlib.h>
#include "checktools.inc"

#include "clusterlst.h"
#include <stdio.h>                              /* perror */

/** Dummy test to trigger fixtures
 *
 * This test is defined and added to all testcases in order to execute at
 * least one test test in each testcase of the testsuite, trigger the
 * fixtures and have a decent coverage report.
 */
START_TEST(clusterlst_test) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    abort();
}
END_TEST

/* Test variables used in setups, teardowns and tests */
void       (*func_clusterlist_add)(cluster_t* cluster)=NULL;
cluster_t* (*func_clusterlist_find) (const char* host)=NULL;
cluster_t* (*func_clusterlist_first)()=NULL;
cluster_t* (*func_clusterlist_next)()=NULL;
cluster_t* (*func_clusterlist_get)()=NULL;
cluster_t* ptr_cluster=NULL;

START_TEST(clusterlst_uninitialized_add) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    /* Currently leaked */
    if (NULL==(ptr_cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    ptr_cluster->host=NULL;
    ptr_cluster->user=NULL;
    ptr_cluster->pass=NULL;
    ptr_cluster->insecure=NULL;
    ptr_cluster->cacert=NULL;
    clusterlist_add(ptr_cluster);

    /* Check functors update by init */
    ck_assert(func_clusterlist_add!=clusterlist_add);
    ck_assert(func_clusterlist_find!=clusterlist_find);
    ck_assert(func_clusterlist_first!=clusterlist_first);
    ck_assert(func_clusterlist_next!=clusterlist_next);
    ck_assert(func_clusterlist_get!=clusterlist_get);

    /* Second run should not invoke init and change the pointers and functors */
    func_clusterlist_add=clusterlist_add;
    func_clusterlist_find=clusterlist_find;
    func_clusterlist_first=clusterlist_first;
    func_clusterlist_next=clusterlist_next;
    func_clusterlist_get=clusterlist_get;

    /* No need for another cluster data, reuse the same */
    clusterlist_add(ptr_cluster);

    ck_assert(func_clusterlist_add==clusterlist_add);
    ck_assert(func_clusterlist_find==clusterlist_find);
    ck_assert(func_clusterlist_first==clusterlist_first);
    ck_assert(func_clusterlist_next==clusterlist_next);
    ck_assert(func_clusterlist_get==clusterlist_get);
}
END_TEST

START_TEST(clusterlst_uninitialized_find) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    ptr_cluster=clusterlist_find("");
    ck_assert(ptr_cluster==NULL);

    /* Check functors update by init */
    ck_assert(func_clusterlist_add!=clusterlist_add);
    ck_assert(func_clusterlist_find!=clusterlist_find);
    ck_assert(func_clusterlist_first!=clusterlist_first);
    ck_assert(func_clusterlist_next!=clusterlist_next);
    ck_assert(func_clusterlist_get!=clusterlist_get);

    /* Second run should not invoke init and change the pointers and functors */
    func_clusterlist_add=clusterlist_add;
    func_clusterlist_find=clusterlist_find;
    func_clusterlist_first=clusterlist_first;
    func_clusterlist_next=clusterlist_next;
    func_clusterlist_get=clusterlist_get;

    ptr_cluster=clusterlist_find("");
    ck_assert(ptr_cluster==NULL);

    ck_assert(func_clusterlist_add==clusterlist_add);
    ck_assert(func_clusterlist_find==clusterlist_find);
    ck_assert(func_clusterlist_first==clusterlist_first);
    ck_assert(func_clusterlist_next==clusterlist_next);
    ck_assert(func_clusterlist_get==clusterlist_get);
}
END_TEST

START_TEST(clusterlst_uninitialized_first) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    ptr_cluster=clusterlist_first();
    ck_assert(ptr_cluster==NULL);

    /* Check functors update by init */
    ck_assert(func_clusterlist_add!=clusterlist_add);
    ck_assert(func_clusterlist_find!=clusterlist_find);
    ck_assert(func_clusterlist_first!=clusterlist_first);
    ck_assert(func_clusterlist_next!=clusterlist_next);
    ck_assert(func_clusterlist_get!=clusterlist_get);

    /* Second run should not invoke init and change the pointers and functors */
    func_clusterlist_add=clusterlist_add;
    func_clusterlist_find=clusterlist_find;
    func_clusterlist_first=clusterlist_first;
    func_clusterlist_next=clusterlist_next;
    func_clusterlist_get=clusterlist_get;

    ptr_cluster=clusterlist_first();
    ck_assert(ptr_cluster==NULL);

    ck_assert(func_clusterlist_add==clusterlist_add);
    ck_assert(func_clusterlist_find==clusterlist_find);
    ck_assert(func_clusterlist_first==clusterlist_first);
    ck_assert(func_clusterlist_next==clusterlist_next);
    ck_assert(func_clusterlist_get==clusterlist_get);
}
END_TEST

START_TEST(clusterlst_uninitialized_next) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    ptr_cluster=clusterlist_next();
    ck_assert(ptr_cluster==NULL);

    /* Check functors update by init */
    ck_assert(func_clusterlist_add!=clusterlist_add);
    ck_assert(func_clusterlist_find!=clusterlist_find);
    ck_assert(func_clusterlist_first!=clusterlist_first);
    ck_assert(func_clusterlist_next!=clusterlist_next);
    ck_assert(func_clusterlist_get!=clusterlist_get);

    /* Second run should not invoke init and change the pointers and functors */
    func_clusterlist_add=clusterlist_add;
    func_clusterlist_find=clusterlist_find;
    func_clusterlist_first=clusterlist_first;
    func_clusterlist_next=clusterlist_next;
    func_clusterlist_get=clusterlist_get;

    ptr_cluster=clusterlist_next();
    ck_assert(ptr_cluster==NULL);

    ck_assert(func_clusterlist_add==clusterlist_add);
    ck_assert(func_clusterlist_find==clusterlist_find);
    ck_assert(func_clusterlist_first==clusterlist_first);
    ck_assert(func_clusterlist_next==clusterlist_next);
    ck_assert(func_clusterlist_get==clusterlist_get);
}
END_TEST

START_TEST(clusterlst_uninitialized_get) {
    /* Need to have a forked env to start uninitialized */
    forktest_only;

    ptr_cluster=clusterlist_get();
    ck_assert(ptr_cluster==NULL);

    /* Check functors update by init */
    ck_assert(func_clusterlist_add!=clusterlist_add);
    ck_assert(func_clusterlist_find!=clusterlist_find);
    ck_assert(func_clusterlist_first!=clusterlist_first);
    ck_assert(func_clusterlist_next!=clusterlist_next);
    ck_assert(func_clusterlist_get!=clusterlist_get);

    /* Second run should not invoke init and change the pointers and functors */
    func_clusterlist_add=clusterlist_add;
    func_clusterlist_find=clusterlist_find;
    func_clusterlist_first=clusterlist_first;
    func_clusterlist_next=clusterlist_next;
    func_clusterlist_get=clusterlist_get;

    ptr_cluster=clusterlist_get();
    ck_assert(ptr_cluster==NULL);

    ck_assert(func_clusterlist_add==clusterlist_add);
    ck_assert(func_clusterlist_find==clusterlist_find);
    ck_assert(func_clusterlist_first==clusterlist_first);
    ck_assert(func_clusterlist_next==clusterlist_next);
    ck_assert(func_clusterlist_get==clusterlist_get);
}
END_TEST

START_TEST(clusterlst_initialized_add) {
    /* Need to have a forked env to start initialized and empty */
    forktest_only;

    /* Currently leaked */
    if (NULL==(ptr_cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    ptr_cluster->host=NULL;
    ptr_cluster->user=NULL;
    ptr_cluster->pass=NULL;
    ptr_cluster->insecure=NULL;
    ptr_cluster->cacert=NULL;
    clusterlist_add(ptr_cluster);

    ck_assert(func_clusterlist_add==clusterlist_add);
    ck_assert(func_clusterlist_find==clusterlist_find);
    ck_assert(func_clusterlist_first==clusterlist_first);
    ck_assert(func_clusterlist_next==clusterlist_next);
    ck_assert(func_clusterlist_get==clusterlist_get);
}
END_TEST

START_TEST(clusterlst_initialized_find) {
    /* Need to have a forked env to start initialized and empty */
    forktest_only;

    /* Find in empty list */
    ck_assert(NULL==clusterlist_find(NULL));
    ck_assert(func_clusterlist_add==clusterlist_add);
    ck_assert(func_clusterlist_find==clusterlist_find);
    ck_assert(func_clusterlist_first==clusterlist_first);
    ck_assert(func_clusterlist_next==clusterlist_next);
    ck_assert(func_clusterlist_get==clusterlist_get);

    ck_assert(NULL==clusterlist_find(""));
    ck_assert(func_clusterlist_add==clusterlist_add);
    ck_assert(func_clusterlist_find==clusterlist_find);
    ck_assert(func_clusterlist_first==clusterlist_first);
    ck_assert(func_clusterlist_next==clusterlist_next);
    ck_assert(func_clusterlist_get==clusterlist_get);

    ck_assert(NULL==clusterlist_find("a"));
    ck_assert(func_clusterlist_add==clusterlist_add);
    ck_assert(func_clusterlist_find==clusterlist_find);
    ck_assert(func_clusterlist_first==clusterlist_first);
    ck_assert(func_clusterlist_next==clusterlist_next);
    ck_assert(func_clusterlist_get==clusterlist_get);
}
END_TEST

START_TEST(clusterlst_initialized_firstnextget) {
    /* Need to have a forked env to start initialized and empty */
    forktest_only;

    ptr_cluster = clusterlist_first();
    while (ptr_cluster) {
        ptr_cluster = clusterlist_next();
        ck_assert(clusterlist_get()==ptr_cluster);
    }
    ck_assert(NULL==clusterlist_next());
    ck_assert(NULL==clusterlist_get());
}
END_TEST

START_TEST(clusterlst_populated_find) {
    /* Find in NULL,"","a","b","",NULL */
    /* Find first, middle, last, nonexistant, duplicate */
    ck_assert(NULL!=clusterlist_find(NULL));
    ck_assert(NULL!=clusterlist_find(""));
    ck_assert(NULL!=clusterlist_find("a"));
    ck_assert(NULL==clusterlist_find("c"));
}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void clusterlst_checked_uninitialized_setup() {
    signals_catch();
    forktest_gprofdir();

    func_clusterlist_add=clusterlist_add;
    func_clusterlist_find=clusterlist_find;
    func_clusterlist_first=clusterlist_first;
    func_clusterlist_next=clusterlist_next;
    func_clusterlist_get=clusterlist_get;
}

/* Begining of test (potentially in child) */
void clusterlst_checked_initialized_setup() {
    signals_catch();
    forktest_gprofdir();

    clusterlist_get();
    func_clusterlist_add=clusterlist_add;
    func_clusterlist_find=clusterlist_find;
    func_clusterlist_first=clusterlist_first;
    func_clusterlist_next=clusterlist_next;
    func_clusterlist_get=clusterlist_get;
}

/* Begining of test (potentially in child) */
void clusterlst_checked_populated_setup() {

    signals_catch();
    forktest_gprofdir();

    /* Currently leaked */
    if (NULL==(ptr_cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    ptr_cluster->host=NULL;
    clusterlist_add(ptr_cluster);

    /* Currently leaked */
    if (NULL==(ptr_cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    ptr_cluster->host=strdup("");
    clusterlist_add(ptr_cluster);

    /* Currently leaked */
    if (NULL==(ptr_cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    ptr_cluster->host=strdup("a");
    clusterlist_add(ptr_cluster);

    /* Currently leaked */
    if (NULL==(ptr_cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    ptr_cluster->host=strdup("b");
    clusterlist_add(ptr_cluster);

    /* Currently leaked */
    if (NULL==(ptr_cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    ptr_cluster->host=strdup("");
    clusterlist_add(ptr_cluster);

    /* Currently leaked */
    if (NULL==(ptr_cluster=malloc(sizeof(struct cluster_s)))) {
        perror("clusterlst_uninitialized_add");
        abort();
    }
    ptr_cluster->host=NULL;
    clusterlist_add(ptr_cluster);

    func_clusterlist_add=clusterlist_add;
    func_clusterlist_find=clusterlist_find;
    func_clusterlist_first=clusterlist_first;
    func_clusterlist_next=clusterlist_next;
    func_clusterlist_get=clusterlist_get;
}

/* End of test (potentially in child) */
void clusterlst_checked_uninitialized_teardown() {
    signals_release();
}
/************************************************************************/
void clusterlst_unchecked_common_setup() {
    forktest_init();
}
void clusterlst_unchecked_common_teardown() {
}
/************************************************************************/
Suite* clusterlst_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("Clusterlst");

    tc = tcase_create("ClusterlstUninitialized");
    tcase_set_tags(tc,"ClusterlstTag ClusterlstTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, clusterlst_checked_uninitialized_setup, clusterlst_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, clusterlst_unchecked_common_setup, clusterlst_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, clusterlst_test,6);
    tcase_add_test(tc, clusterlst_uninitialized_add);
    tcase_add_test(tc, clusterlst_uninitialized_find);
    tcase_add_test(tc, clusterlst_uninitialized_first);
    tcase_add_test(tc, clusterlst_uninitialized_next);
    tcase_add_test(tc, clusterlst_uninitialized_get);

    tc = tcase_create("ClusterlstInitialized");
    tcase_set_tags(tc,"ClusterlstTag ClusterlstTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, clusterlst_checked_initialized_setup, clusterlst_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, clusterlst_unchecked_common_setup, clusterlst_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, clusterlst_test,6);
    tcase_add_test(tc, clusterlst_initialized_add);
    tcase_add_test(tc, clusterlst_initialized_find);
    tcase_add_test(tc, clusterlst_initialized_firstnextget);

    tc = tcase_create("ClusterlstPopulated");
    tcase_set_tags(tc,"ClusterlstTag ClusterlstTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, clusterlst_checked_populated_setup, clusterlst_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, clusterlst_unchecked_common_setup, clusterlst_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, clusterlst_test,6);
    tcase_add_test(tc, clusterlst_initialized_add);
    tcase_add_test(tc, clusterlst_populated_find);
    tcase_add_test(tc, clusterlst_initialized_firstnextget);

    return s;
}
/* vim: set tw=80: */
