/**   @file  suite_json.c
 *   @brief  Check testsuite template
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-11-06 11:23
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <check.h>
#include <stdlib.h>
#include <stdio.h>                              /* fprintf */
#include "checktools.inc"

#include "json.h"

/** Dummy test to trigger fixtures
 *
 * This test is defined and added to all testcases in order to execute at
 * least one test test in each testcase of the testsuite, trigger the
 * fixtures and have a decent coverage report.
 */
START_TEST(json_test) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    abort();
}
END_TEST

int testjsontext(char* jsonstr, char* expectedstr) {
    cJSON* val_json = cJSON_Parse(jsonstr);
    char* val_text=NULL;
    int retval;
    val_text = json2text(val_json);
    retval=strcmp(val_text,expectedstr);
    if (retval) fprintf(stderr, "%s -> %s (not %s)\n",jsonstr, val_text, expectedstr);
    free(val_text);
    cJSON_Delete(val_json);
    return retval;
}

START_TEST(json_json2text) {
    cJSON* json;
    char* text;

    json=NULL;
    text=NULL;
    text = json2text(json);
    ck_assert(0==strcmp(text,"Invalid data"));
    free(text);

    json = cJSON_CreateObject();
    text=NULL;
    text = json2text(json);
    ck_assert(0==strcmp(text,"{}"));
    free(text);
    cJSON_Delete(json);

    ck_assert(0==testjsontext("{}","{}"));
    ck_assert(0==testjsontext("[]","[]"));
    ck_assert(0==testjsontext("\"ab\"","ab"));
    ck_assert(0==testjsontext("\"1\"","1"));
    ck_assert(0==testjsontext("1","1"));
    ck_assert(0==testjsontext("null","null"));
    ck_assert(0==testjsontext("true","true"));
    ck_assert(0==testjsontext("false","false"));
    ck_assert(0==testjsontext("a","Invalid data"));
    ck_assert(0==testjsontext("","Invalid data"));
}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void json_checked_uninitialized_setup() {
    signals_catch();
    forktest_gprofdir();
}

/* End of test (potentially in child) */
void json_checked_uninitialized_teardown() {
    signals_release();
}
/************************************************************************/
void json_unchecked_common_setup() {
    forktest_init();
}
void json_unchecked_common_teardown() {
}
/************************************************************************/
Suite* json_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("JSON");

    tc = tcase_create("json2text");
    tcase_set_tags(tc,"JSONTag JSONTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, json_checked_uninitialized_setup, json_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, json_unchecked_common_setup, json_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, json_test,6);
    tcase_add_test(tc, json_json2text);

    return s;
}
/* vim: set tw=80: */
