/**   @file  suite_sclist.c
 *   @brief  Check testsuite template
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  20/03/2022
 *      Revision:  none
 * Last modified:  2024-11-25 09:01
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <check.h>
#include <unistd.h>                             /* pid_t getpid() */
#include <sys/stat.h>                           /* mkdir(), chdir(), mode_t */
#include <stdio.h>                              /* fprintf */
#include <stdlib.h>                             /* abort() */
#include "checktools.inc"

#include "sclist.c"

/** Dummy test to trigger fixtures
 *
 * This test is defined and added to all testcases in order to execute at
 * least one test test in each testcase of the testsuite, trigger the
 * fixtures and have a decent coverage report.
 */
START_TEST(sclist_test) {
    /* This test is supposed to trigger a SIGABRT(6) and will crash the
     * whole testsuite if not caught or not in a child process */
    forktest_only;

    abort();
}
END_TEST

START_TEST(sclist_scenarized_test) {
    sclist_t* sclist=NULL;
    sclistrecord_t* sclistrecord;
    char* txt_a="a";
    char* txt_b="b";

    sclist = sclist_new();
    ck_assert(sclist);
    ck_assert(NULL==sclist->first);
    ck_assert(NULL==sclist->last);

    sclistrecord = sclist_firstrecord(sclist);
    ck_assert(sclistrecord == sclist->first);

    sclistrecord = sclist_addrecord(sclist,txt_a);
    ck_assert(sclistrecord!=NULL);
    ck_assert(sclist->last==sclistrecord);
    ck_assert(sclist->first==sclistrecord);
    ck_assert(sclist->first->value==txt_a);
    ck_assert(sclist->first->next==NULL);

    sclistrecord = sclist_addrecord(sclist,txt_b);
    ck_assert(sclistrecord!=NULL);
    ck_assert(sclist->last==sclistrecord);
    ck_assert(sclist->first!=sclistrecord);
    ck_assert(sclist->first->value==txt_a);
    ck_assert(sclist->first->next==sclistrecord);
    ck_assert(sclist->first->next->next==NULL);
    ck_assert(sclist->first->next->value==txt_b);

    /* Duplicate */
    sclistrecord = sclist_addrecord(sclist,txt_a);
    ck_assert(sclistrecord!=NULL);
    ck_assert(sclist->last==sclistrecord);
    ck_assert(sclist->first!=sclistrecord);
    ck_assert(sclist->first->value==txt_a);
    ck_assert(sclist->first->next!=sclistrecord);
    ck_assert(sclist->first->next->value==txt_b);
    ck_assert(sclist->first->next->next==sclistrecord);
    ck_assert(sclist->first->next->next->value==txt_a);
    ck_assert(sclist->first->next->next->next==NULL);

    sclistrecord = sclist_firstrecord(sclist);
    ck_assert(sclistrecord == sclist->first);
    ck_assert(sclistrecord->value = txt_a);
    ck_assert(sclist_getvalue(sclistrecord)==txt_a);
    ck_assert(sclistrecord->next != NULL);

    sclistrecord = sclist_nextrecord(sclistrecord);
    ck_assert(sclistrecord = sclist->first->next);
    ck_assert(sclistrecord->value = txt_b);
    ck_assert(sclist_getvalue(sclistrecord)==txt_b);
    ck_assert(sclistrecord->next != NULL);

    sclistrecord = sclist_nextrecord(sclistrecord);
    ck_assert(sclistrecord = sclist->first->next->next);
    ck_assert(sclistrecord->value = txt_a);
    ck_assert(sclist_getvalue(sclistrecord)==txt_a);
    ck_assert(sclistrecord->next == NULL);

    sclistrecord = sclist_nextrecord(sclistrecord);
    ck_assert(sclistrecord == sclist->first->next->next->next);
    ck_assert(sclistrecord == NULL);

    sclistrecord = sclist->last;
    ck_assert(sclist==sclist_remrecord(sclist, sclistrecord));
    ck_assert(sclist->first!=NULL);
    ck_assert(sclist->first->value==txt_a);
    ck_assert(sclist->first->next!=NULL);
    ck_assert(sclist->first->next->value==txt_b);
    ck_assert(sclist->first->next->next==NULL);
    ck_assert(sclist->last!=NULL);
    ck_assert(sclist->last->value==txt_b);
    ck_assert(sclist->last->next==NULL);

    sclistrecord = sclist->first->next;
    ck_assert(sclist==sclist_remrecord(sclist, sclistrecord));
    ck_assert(sclist->first!=NULL);
    ck_assert(sclist->first->value==txt_a);
    ck_assert(sclist->first->next==NULL);
    ck_assert(sclist->last!=NULL);
    ck_assert(sclist->last->value==txt_a);
    ck_assert(sclist->last->next==NULL);

    sclistrecord = sclist->first;
    ck_assert(sclist==sclist_remrecord(sclist, sclistrecord));
    ck_assert(sclist->first==NULL);
    ck_assert(sclist->last==NULL);

    sclist_del(sclist);
}
END_TEST

START_TEST(sclist_remrecord_notfound) {
    sclist_t* sclist;
    sclistrecord_t* sclistrecord;
    char* txt_a="a";
    char* txt_b="b";

    sclist = sclist_new();
    sclist_addrecord(sclist,txt_a);
    sclistrecord = sclist_addrecord(sclist,txt_b);
    ck_assert(sclist==sclist_remrecord(sclist, sclistrecord));

    /* Failure expected, record not found */
    ck_assert(NULL==sclist_remrecord(sclist, sclistrecord));

    sclist_del(sclist);
}
END_TEST

START_TEST(sclist_remrecord_notfound_empty) {
    forktest_only;

    sclist_t* sclist;
    sclistrecord_t* sclistrecord;
    char* txt_a="a";

    sclist = sclist_new();
    sclistrecord = sclist_addrecord(sclist,txt_a);
    ck_assert(sclist==sclist_remrecord(sclist, sclistrecord));

    /* Failure expected, record not found */
    ck_assert(NULL==sclist_remrecord(sclist, sclistrecord));

    sclist_del(sclist);
}
END_TEST

START_TEST(sclist_del_test) {
    sclist_t* sclist;
    char* txt_a="a";
    char* txt_b="b";

    sclist = sclist_new();
    sclist_addrecord(sclist,txt_a);
    sclist_addrecord(sclist,txt_b);
    sclist_del(sclist);
}
END_TEST

START_TEST(sclist_del_empty) {
    sclist_t* sclist;

    sclist = sclist_new();
    sclist_del(sclist);
}
END_TEST

/************************************************************************/
/* Begining of test (potentially in child) */
void sclist_checked_uninitialized_setup() {
    signals_catch();
    forktest_gprofdir();
}

/* End of test (potentially in child) */
void sclist_checked_uninitialized_teardown() {
    signals_release();
}
/************************************************************************/
void sclist_unchecked_common_setup() {
    forktest_init();
}
void sclist_unchecked_common_teardown() {
}
/************************************************************************/
Suite* sclist_suite() {
    Suite *s;
    TCase *tc;

    s = suite_create("SCListSuite");

    tc = tcase_create("SCListCase");
    tcase_set_tags(tc,"SCListTag SCListTag2");
    /* tcase_set_timeout(tc,5); */ /* seconds */
    tcase_add_checked_fixture(tc, sclist_checked_uninitialized_setup, sclist_checked_uninitialized_teardown);
    tcase_add_unchecked_fixture(tc, sclist_unchecked_common_setup, sclist_unchecked_common_teardown);
    suite_add_tcase(s, tc);
    tcase_add_test_raise_signal(tc, sclist_test,6);
    tcase_add_test(tc, sclist_scenarized_test);
    tcase_add_test(tc, sclist_remrecord_notfound);
    tcase_add_test(tc, sclist_remrecord_notfound_empty);
    tcase_add_test(tc, sclist_del_test);
    tcase_add_test(tc, sclist_del_empty);

    return s;
}
/* vim: set tw=80: */
